<?php
namespace Geral\Logic\Login;

use \Geral\Entities\Filial;
use \Geral\Entities\Filioper;
use \Geral\Exceptions;

/**
 *
 */
class LoginEstrutura
{

    private $conn;

    private $email;
    private $senha;
    private $cdfilial;
    private $nrorganizacao;
    private $nrestrutura;

    /** @var $organizacao \Geral\Entities\Organizacao */
    private $organizacao;
    /** @var $filial \Geral\Entities\Filial*/
    private $filial;
    /** @var $operador \OperadorEntity */
    private $operador;
    /** @var $estrutura \Geral\Entities\Estruturam */
    private $estrutura;


    /**
     * Construtor.
     *
     * @param $email
     * @param $senha
     * @param $cdfilial
     * @param $nmorganizacao
     */
    public function __construct($email, $senha, $cdfilial, $nmorganizacao)
    {
        $this->email         = $email;
        $this->senha         = $senha;
        $this->cdfilial      = $cdfilial;
        $this->nrorganizacao = $nmorganizacao;
    }

    /**
     * Inicializa os objetos de orgnaizacao, filial e operador, alem de validar os dados do usuario.
     *
     * @return bool
     * @throws \Exception
     */
    public function valida()
    {
        $this->validaOperador();
        $this->obterParametros();
        $this->validaOrganizacaoEstruturaDoOperador();
        $this->obterEstrutura();
        $this->obterOrganizacao();

        if (\Setforms2\Shared\Config::getInstance()->getProjectUsafilial() == 'true') {
            $this->obterFilial();
        }
        return true;
    }

    /**
     * Retorna o objeto de operador.
     *
     * @return \Geral\Entities\Operador
     */
    public function getOperador()
    {
        return $this->operador;
    }

    /**
     * Retorna o objeto de filial.
     *
     * @return \Geral\Entities\Filial
     */
    public function getFilial()
    {
        return $this->filial;
    }

    /**
     * Retorna o objeto de organizacao.
     *
     * @return \Geral\Entities\Organizacao
     */
    public function getOrganizacao()
    {
        return $this->organizacao;
    }

    /**
     * Retorna o objeto de organizacao a partir do nome da organizacao ou do nrorg. Caso este nao exista, o metodo lanca
     * excessao.
     *
     * @throws \Geral\Exceptions\OrganizacaoNaoEncontrada
     * @return \Geral\Entities\Organizacao
     */
    private function obterOrganizacao()
    {
        $this->organizacao = \Geral\Entities\Organizacao::findOneByNrorg($this->nrorganizacao);
        if (!$this->organizacao) {
            throw new \Geral\Exceptions\OrganizacaoNaoEncontrada();
        }
    }

    /**
     * Retorna o objeto de filial a partir do codigo da filial. Caso nao encontre uma filial, retorna uma mensagem de
     * erro.
     *
     * @throws \Geral\Exceptions\FilialNaoEncontrada
     * @return Filial
     */
    private function obterFilial()
    {
        /** @var $filial \Geral\Entities\Filial */
        $this->filial = Filial::findOneByCdfilialAndNrorg(
            $this->estrutura->getCdintestrutura(), $this->organizacao->getNrorg()
        );

        if (!$this->filial) {
            throw new \Geral\Exceptions\FilialNaoEncontrada();
        }
    }

    /**
     * Retorna o objeto de operador a partir do email
     *
     * @return \Geral\Entities\Operador
     */
    private function obterOperador()
    {
        /** @var $operador \Geral\Entities\Operador */
        $this->operador = \Geral\Entities\Operador::findOneByDsemailoper($this->email);
        return $this->operador;
    }

    /**
     * Valida o operador, atualiza a senha se for primeiro login, verifica os parametros do operador.
     *
     * @throws \LoginException
     * @throws \Geral\Exceptions\OperadorNaoEncontrado
     */
    private function validaOperador()
    {
        $this->loginVirtualPrivateDatabase();

        $this->operador = $this->obterOperador();
        if (!is_object($this->operador)) {
            throw new Exceptions\OperadorNaoEncontrado();
        }

        // Se operador tem senha para o sistema Delphi, sera essa a senha considerada.
        if ($this->operador->getCdsenhoper() && !$this->operador->getCdsenhaoperweb()) {
            if ($msg = $this->_validaOperadorWebService()) {
                throw new \LoginException($msg);
            }
        } else if ($this->_validaAlteraSenha() === false) {
            throw new \Exception('Senha inválida!'); // Senha Invalida.
        }
    }

    public function getUserLoginConnection() {
        if (!$this->conn) {
            $tns = "(DESCRIPTION =
        (ADDRESS_LIST =
          (ADDRESS = (PROTOCOL = TCP)(HOST = 192.168.122.42)(PORT = 1521))
        )
        (CONNECT_DATA =
          (SERVICE_NAME = oracle)
        )
      )";
            $conn = new \Doctrine2\DBAL\Driver\PDOConnection("oci:dbname=$tns", 'user_login', 'teknisa');
            $this->conn = $conn;
        }
        return $this->conn;
    }

    /**
     * Valida operador usando web service para execucao de metodo Delphi.
     *
     * @throws \Exception
     * @return null|string
     */
    private function _validaOperadorWebService() {
        $url = 'https://www.security.teknisa.com/comwebservice/service.asmx?wsdl';

        $opcoes = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $argumentos = array(
            new \SoapVar($this->operador->getCdoperador(), XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($this->senha, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );

        $parametros = array(
            'rdm'    => 'Bib_Html.rdmDBGeralHtm',
            'method' => 'ValidaConexaoHTM',
            'args'   => $argumentos,
        );
        try {
            $client = new \SoapClient($url, $opcoes);
            $retorno = $client->executeCOMMethod($parametros);
            $resultado = $retorno->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
        if ($resultado === 'SenhaInv') {
            return 'msg_valida_senha_operador'; // Senha Invalida.
        }

        if ($resultado === 'OperNCad') {
            return 'msg_operador_n_cadastrado'; // Operador nao cadastrado.
        }
        return null;
    }

    /**
     *  Valida ou altera senha.
     *
     * @return void|boolean
     */
    private function _validaAlteraSenha(){
        $criptosenha    = md5($this->senha);
        $cdsenhaoperweb = ($this->senha !== '')?$this->operador->getCdsenhaoperweb() : null;

        if (($cdsenhaoperweb === null || $cdsenhaoperweb === 'd41d8cd98f00b204e9800998ecf8427e') && $this->senha !== '') {
            \Operador::alteraSenhaOperador($this->operador->getCdoperador(), $criptosenha);
            return null;
        } else if ($cdsenhaoperweb !== $criptosenha) {
            return false;
        }
        return true;
    }

    /**
     * @throws \Exception
     */
    private function validaOrganizacaoEstruturaDoOperador()
    {
        if (!$this->nrorganizacao || (\Setforms2\Shared\Config::getInstance()->getProjectParamestrutura() == 'true' &&
            !$this->nrestrutura))
        {
            throw new \Exception('Operador sem parametrizacao de organizacao ou estrutura');
        }
    }

    /**
     * @throws \Geral\Logic\Login\Exceptions\OperadorSemParametros
     * @return \Geral\Entities\GpeParamoper
     */
    private function obterParametros()
    {
        /** @var $paramoperador \Geral\Entities\GpeParamoper */
        $paramoperador = \Geral\Entities\GpeParamoper::findOneByNrorgAndCdoperador(
            $this->operador->getNrorg(), $this->operador->getCdoperador()
        );
        if (!$paramoperador) {
            throw new \Geral\Logic\Login\Exceptions\OperadorSemParametros();
        }
        $this->nrorganizacao = $paramoperador->getNrorgtrab();
        $this->nrestrutura = $paramoperador->getNrestruttrab();
    }

    private function obterEstrutura()
    {
        $this->estrutura = \Geral\Entities\Estruturam::findOneByNrestruturamAndNrorg(
            $this->nrestrutura, $this->nrorganizacao
        );
    }

    /**
     * @throws \Exception
     * @return mixed
     */
    private function loginVirtualPrivateDatabase()
    {
        if (\Setforms2\Shared\Config::getInstance()->getProjectUsavpd() != 'true') {
            return;
        }
        $conn = $this->getUserLoginConnection();
        $sql = "SELECT * FROM OPERADOR WHERE DSEMAILOPER = '{$this->email}'";
        $operador = $conn->query($sql)->fetch(\Doctrine2\DBAL\Driver\PDOConnection::FETCH_ASSOC);;
        $cdoperador = $operador['CDOPERADOR'];
        if (!$cdoperador) {
            throw new \Exception('Operador não encontrado.');
        }
        $sql = "SELECT NRORGTRAB FROM GPE_PARAMOPER WHERE CDOPERADOR = '$cdoperador'";
        $paramoper = $conn->query($sql)->fetch(\Doctrine2\DBAL\Driver\PDOConnection::FETCH_ASSOC);
        if (!$paramoper) {
            throw new \Exception('Parâmetro de operador não encontrado.');
        }
        $this->nrorganizacao = $paramoper['NRORGTRAB'];
        if (!$this->nrorganizacao) {
            throw new \Exception('Organização de trabalho não parametrizada, contacte o administrador.');
        }

        \Setforms2\Shared\Config::getInstance()->setConnectionUsername('USR_ORG_' . $this->nrorganizacao);
        \Setforms2\Shared\Config::getInstance()->setConnectionPassword(\Setforms2\Shared\Util\Util::encrypt('teknisa'));
    }

}