-- ============================================================
--   TABLE: PRODLOJANIVEL
-- ============================================================
CREATE TABLE PRODLOJANIVEL (
   CDFILIAL       VARCHAR2(4)  NOT NULL,
   CDLOJA         VARCHAR2(10) NOT NULL,
   CDPRODUTO      VARCHAR2(10) NOT NULL,
   NRSEQIMPRPROD  VARCHAR2(5)  NULL,
   NRSEQIMPRPROD2 VARCHAR2(5)  NULL,
   NRSEQIMPRPUXA  VARCHAR2(5)  NULL,
   IDIMPCANITEM   VARCHAR2(1)  DEFAULT 'N' NOT NULL
		CONSTRAINT CKC_IDIMPCANITEM_PRODLOJANIVEL CHECK (IDIMPCANITEM IN ('S','N')),
   CDSETOR        VARCHAR2(4)  NULL,
   CDPRODEXCECAO  VARCHAR2(10) NULL,
   NRORG          NUMBER       DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NOT NULL,
   IDATIVO	  VARCHAR2(1)  DEFAULT 'S' NOT NULL,
		CONSTRAINT CKC_IDATIVO_PRODLOJANIVEL CHECK (IDATIVO IN ('S','N')),          
   DTINCLUSAO     DATE         NULL,
   DTULTATU       DATE         NULL,
   NRORGINCLUSAO  NUMBER       DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NULL,
   CDOPERINCLUSAO VARCHAR2(12) DEFAULT '1' NULL,
   NRORGULTATU    NUMBER       DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NULL,
   CDOPERULTATU   VARCHAR2(12) DEFAULT '1' NULL,
		CONSTRAINT PK_PRODLOJANIVEL PRIMARY KEY (CDFILIAL, CDLOJA, CDPRODUTO)
)
/

CREATE INDEX VND00811_FK ON PRODLOJANIVEL (CDFILIAL ASC, CDLOJA ASC)
/

ALTER TABLE PRODLOJANIVEL
   ADD CONSTRAINT FKVND00811LOJA FOREIGN KEY (CDFILIAL, CDLOJA)
      REFERENCES LOJA (CDFILIAL, CDLOJA)
/

CREATE INDEX VND00812_FK ON PRODLOJANIVEL (CDPRODUTO ASC)
/

ALTER TABLE PRODLOJANIVEL
   ADD CONSTRAINT FKVND00812PRODUTO FOREIGN KEY (CDPRODUTO)
      REFERENCES PRODUTO (CDPRODUTO)
/

CREATE INDEX VND00813_FK ON PRODLOJANIVEL (CDFILIAL ASC, CDLOJA ASC, NRSEQIMPRPROD ASC)
/

ALTER TABLE PRODLOJANIVEL
    ADD CONSTRAINT FKVND00813IMPRLOJA FOREIGN KEY  (CDFILIAL, CDLOJA, NRSEQIMPRPROD)
       REFERENCES IMPRLOJA (CDFILIAL, CDLOJA, NRSEQIMPRLOJA)
/

CREATE INDEX VND00814_FK ON PRODLOJANIVEL (CDFILIAL ASC, CDLOJA ASC, NRSEQIMPRPROD2 ASC)
/

ALTER TABLE PRODLOJANIVEL
    ADD CONSTRAINT FKVND00814IMPRLOJA FOREIGN KEY  (CDFILIAL, CDLOJA, NRSEQIMPRPROD2)
       REFERENCES IMPRLOJA (CDFILIAL, CDLOJA, NRSEQIMPRLOJA)
/

CREATE INDEX VND00815_FK ON PRODLOJANIVEL (CDFILIAL ASC, CDLOJA ASC, NRSEQIMPRPUXA ASC)
/

ALTER TABLE PRODLOJANIVEL
    ADD CONSTRAINT FKVND00815IMPRLOJA FOREIGN KEY  (CDFILIAL, CDLOJA, NRSEQIMPRPUXA)
       REFERENCES IMPRLOJA (CDFILIAL, CDLOJA, NRSEQIMPRLOJA)
/

CREATE INDEX VND00816_FK ON PRODLOJANIVEL (CDSETOR ASC)
/

ALTER TABLE PRODLOJANIVEL
    ADD CONSTRAINT FKVND00816SETOR FOREIGN KEY  (CDSETOR)
       REFERENCES SETOR (CDSETOR)
/

CREATE INDEX VND00817_FK ON PRODLOJANIVEL (CDPRODEXCECAO ASC)
/

ALTER TABLE PRODLOJANIVEL
   ADD CONSTRAINT FKVND00817PRODUTO FOREIGN KEY (CDPRODEXCECAO)
      REFERENCES PRODUTO (CDPRODUTO)
/

-- ============================================================
--   TABLE: PRODLOJANIVELESP
-- ============================================================
CREATE TABLE PRODLOJANIVELESP (
   CDFILIAL       VARCHAR2(4)  NOT NULL,
   CDLOJA         VARCHAR2(10) NOT NULL,
   CDPRODUTO      VARCHAR2(10) NOT NULL,
   NRSEQIMPRPROD  VARCHAR2(5)  NULL,
   NRSEQIMPRPROD2 VARCHAR2(5)  NULL,
   NRSEQIMPRPUXA  VARCHAR2(5)  NULL,
   IDIMPCANITEM   VARCHAR2(1)  DEFAULT 'N' NOT NULL
		CONSTRAINT CKC_IDIMPCANITEM_PRDLJNIVELESP CHECK (IDIMPCANITEM IN ('S','N')),
   CDSETOR        VARCHAR2(4)  NULL,
   CDPRODEXCECAO  VARCHAR2(10) NULL,
   NRORG          NUMBER       DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NOT NULL,
   IDATIVO	  VARCHAR2(1)  DEFAULT 'S' NOT NULL,
		CONSTRAINT CKC_IDATIVO_PRDLJNIVELESP CHECK (IDATIVO IN ('S','N')),          
   DTINCLUSAO     DATE         NULL,
   DTULTATU       DATE         NULL,
   NRORGINCLUSAO  NUMBER       DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NULL,
   CDOPERINCLUSAO VARCHAR2(12) DEFAULT '1' NULL,
   NRORGULTATU    NUMBER       DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NULL,
   CDOPERULTATU   VARCHAR2(12) DEFAULT '1' NULL,
		CONSTRAINT PK_PRODLOJANIVELESP PRIMARY KEY (CDFILIAL, CDLOJA, CDPRODUTO)
)
/

-- ============================================================
--   TABLE: PRODUTOESP
-- ============================================================
CREATE TABLE PRODUTOESP (
   CDPRODUTO      VARCHAR2(10) NOT NULL,
   CDARVPROD      VARCHAR2(20) NOT NULL,
   IDPRODATIVPR   VARCHAR2(01),
   NRORG          NUMBER       DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NOT NULL,
   IDATIVO	  VARCHAR2(1)  DEFAULT 'S' NOT NULL,
		CONSTRAINT CKC_IDATIVO_PRODUTOESP CHECK (IDATIVO IN ('S','N')),
   DTINCLUSAO     DATE         NULL,
   DTULTATU       DATE         NULL,
   NRORGINCLUSAO  NUMBER       DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NULL,
   CDOPERINCLUSAO VARCHAR2(12) DEFAULT '1' NULL,
   NRORGULTATU    NUMBER       DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NULL,
   CDOPERULTATU   VARCHAR2(12) DEFAULT '1' NULL,
      CONSTRAINT PK_PRODUTOESP PRIMARY KEY (CDPRODUTO)
)
/


CREATE INDEX VND00819_FK ON PRODUTOESP(CDPRODUTO ASC)
/

ALTER TABLE PRODUTOESP ADD CONSTRAINT FKVND00819PRODESP_PROD FOREIGN KEY (CDPRODUTO)
   REFERENCES PRODUTO (CDPRODUTO)
/

BEGIN
   FOR C_PROD IN
      (SELECT CDPRODUTO, CDARVPROD, IDPRODATIVPR
         FROM PRODUTO P
        WHERE NOT EXISTS (SELECT 1
                            FROM PRODUTOESP PS
                           WHERE PS.CDPRODUTO = P.CDPRODUTO)) LOOP
      INSERT INTO PRODUTOESP
         (CDPRODUTO, CDARVPROD, IDPRODATIVPR)
      VALUES
         (C_PROD.CDPRODUTO, C_PROD.CDARVPROD, C_PROD.IDPRODATIVPR);
   END LOOP;
   COMMIT;
END;
/